/*
 * Simulator of MCS51 (go.cc)
 *
 * Copyright (C) 1997,97 Drotos Daniel, Talker Bt.
 * 
 * To contact author send email to drdani@mazsola.iit.uni-miskolc.hu
 *
 */

/* This file is part of MCS51 simulator: s51.

S51 is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

S51 is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with S51; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA. */
/*@1@*/

#include "ddconfig.h"

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include "i_string.h"

#include "uccl.h"
#include "globals.h"
#include "dump.h"


/*
 * GO [start [stop]]
 *____________________________________________________________________________
 *
 */

bool
cmd_go(char *cmd, class t_uc *uc)
{
  char *start_str, *stop_str;
  int  start= uc->PC, stop= -1;
  class brkcl *b;
  bool brk_at_stop= FALSE;

  if ((start_str= strtok(NULL, delimiters)) != NULL)
    {
      start= strtol(start_str, NULL, 0);
      if ((stop_str= strtok(NULL, delimiters)) != NULL)
	stop= strtol(stop_str, NULL, 0);
    }
  if (uc->PC != start)
    {
      if (!uc->inst_at(start) &&
	  uc->debug)
	fprintf(uc->cmd_out,
		"Warning: maybe not instruction at %04x\n", start);
      uc->PC= start;
    }
  if (stop >= 0)
    {
      if (start == stop)
	{
	  fprintf(uc->cmd_out, "Addresses must be different.\n");
	  return(FALSE);
	}
      if (b= uc->fbrk_at(stop))
	{
	  brk_at_stop= TRUE;
	  b->cnt= 1;
	}
      else
	{
	  b= new brk_fetch(stop, brkDYNAMIC, 1);
	  uc->fbrk->Insert(b);
	}
    }
  if (uc->fbrk_at(start))
    uc->do_inst(1);
  uc->do_inst(-1);
  if ((stop >= 0) &&
      !brk_at_stop)
    uc->rm_fbrk(stop);
  fprintf(uc->cmd_out, "%d\n", uc->result);
  dump_disass(uc, uc->PC);
  return(FALSE);
}


static void
dump_step(class t_uc *uc)
{
  cmd_dump_regs(NULL, uc);
}


/*
 * STEP [step]
 *____________________________________________________________________________
 *
 */

bool
cmd_step(char *cmd, class t_uc *uc)
{
  int step= 1;
  char *s;

  if ((s= strtok(NULL, delimiters)) != NULL)
    step= strtol(s, NULL, 0);
  while (step--)
    {
      uc->do_inst(1);
      dump_step(uc);
    }
  return(FALSE);
}


/*
 * NEXT [step]
 *____________________________________________________________________________
 *
 */

bool
cmd_next(char *cmd, class t_uc *uc)
{
  int step= 1;
  char *s;
  class brkcl *b;
  uint next;

  if ((s= strtok(NULL, delimiters)) != NULL)
    step= strtol(s, NULL, 0);
  while (step--)
    {
      if ((disass_51[uc->EROM[uc->PC]].branch == 'a') ||
	  (disass_51[uc->EROM[uc->PC]].branch == 'l'))
	{
	  next= uc->PC+2;
	  if (disass_51[uc->EROM[uc->PC]].branch == 'l')
	    next++;
	  if (!uc->fbrk_at(next))
	    {
	      b= new brk_fetch(next, brkDYNAMIC, 1);
	      uc->fbrk->Insert(b);
	    }
	  uc->do_inst(-1);
	}
      else
	uc->do_inst(1);
      dump_step(uc);
    }
  return(FALSE);
}


/*
 * PC [address]
 *____________________________________________________________________________
 *
 */

bool
cmd_pc(char *cmd, class t_uc *uc)
{
  char *s, *p= NULL;
  long pc;

  if ((s= strtok(NULL, delimiters)) == NULL)
    {
      dump_disass(uc, uc->PC);
      return(FALSE);
    }
  pc= strtol(s, &p, 0);
  if (p &&
      ((p == s) ||
       *p))
    fprintf(uc->cmd_out, "Wrong parameter, PC unchanged.\n");
  else
    {
      if (pc >= EROM_SIZE)
	pc= 0;
      if (!uc->inst_at(pc) &&
	  uc->debug)
	fprintf(uc->cmd_out,
		"Warning: maybe not instruction at %06x\n", pc);
      uc->PC= pc;
      dump_disass(uc, uc->PC);
    }
  return(FALSE);
}


/* End of go.cc */
