/*
 * Simulator of MCS51 (mov.cc)
 *
 * Copyright (C) 1997,97 Drotos Daniel, Talker Bt.
 * 
 * To contact author send email to drdani@mazsola.iit.uni-miskolc.hu
 *
 */

/* This file is part of MCS51 simulator: s51.

S51 is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

S51 is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with S51; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA. */
/*@1@*/

#include "ddconfig.h"

#include <stdio.h>

#include "uccl.h"
#include "stypes.h"


/*
 * 0x74 2 12 MOV A,#data
 *____________________________________________________________________________
 *
 */

int
t_uc::inst_mov_a_$data(uchar code)
{
  SFR[event_at.ws= ACC]= fetch();
  return(resGO);
}


/*
 * 0x75 3 24 MOV addr,#data
 *____________________________________________________________________________
 *
 */

int
t_uc::inst_mov_addr_$data(uchar code)
{
  uchar *addr;

  addr= get_direct(fetch(), &event_at.wi, &event_at.ws);
  (*addr)= fetch();
  tick(1);
  proc_write(addr);
  return(resGO);
}


/*
 * 0x76-0x77 2 12 MOV @Ri,#data
 *____________________________________________________________________________
 *
 */

int
t_uc::inst_mov_$ri_$data(uchar code)
{
  uchar *addr;
  int res;

  addr= get_indirect(event_at.wi= *(get_reg(code & 0x01)), &res);
  (*addr)= fetch();
  proc_write(addr);
  return(res);
}


/*
 * 0x78-0x7f 2 12 MOV Rn,#data
 *____________________________________________________________________________
 *
 */

int
t_uc::inst_mov_rn_$data(uchar code)
{
  uchar *reg;

  reg= get_reg(code & 0x07, &event_at.wi);
  (*reg)= fetch();
  return(resGO);
}


/*
 * 0x93 1 24 MOVC A,@A+DPTR
 *____________________________________________________________________________
 *
 */

int
t_uc::inst_movc_a_$a_pc(uchar code)
{
  SFR[ACC]= EROM[event_at.rc= (PC + SFR[ACC]) & (EROM_SIZE - 1)];
  tick(1);
  return(resGO);
}


/*
 * 0x85 3 24 MOV addr,addr
 *____________________________________________________________________________
 *
 */

int
t_uc::inst_mov_addr_addr(uchar code)
{
  uchar *d, *s;

  d= get_direct(fetch(), &event_at.wi, &event_at.ws);
  s= get_direct(fetch(), &event_at.ri, &event_at.rs);
  (*d)= read(s);
  tick(1);
  proc_write(d);
  return(resGO);
}


/*
 * 0x86-0x87 2 24 MOV addr,@Ri
 *____________________________________________________________________________
 *
 */

int
t_uc::inst_mov_addr_$ri(uchar code)
{
  uchar *d, *s;
  int res;

  d= get_direct(fetch(), &event_at.wi, &event_at.ws);
  s= get_indirect(event_at.ri= *(get_reg(code & 0x01)), &res);
  (*d)= *s;
  tick(1);
  proc_write(d);
  return(res);
}


/*
 * 0x88-0x8f 2 24 MOV addr,Rn
 *____________________________________________________________________________
 *
 */

int
t_uc::inst_mov_addr_rn(uchar code)
{
  uchar *addr;

  addr= get_direct(fetch(), &event_at.wi, &event_at.ws);
  (*addr)= *(get_reg(code & 0x07, &event_at.ri));
  proc_write(addr);
  tick(1);
  return(resGO);
}


/*
 * 0x90 3 24 MOV DPTR,#data
 *____________________________________________________________________________
 *
 */

int
t_uc::inst_mov_dptr_$data(uchar code)
{
  SFR[event_at.ws= DPH]= fetch();
  SFR[DPL]= fetch();
  tick(1);
  return(resGO);
}


/*
 * 0x93 1 24 MOVC A,@A+DPTR
 *____________________________________________________________________________
 *
 */

int
t_uc::inst_movc_a_$a_dptr(uchar code)
{
  SFR[ACC]= EROM[event_at.rc= (SFR[DPH]*256+SFR[DPL]+SFR[ACC])&(EROM_SIZE-1)];
  tick(1);
  return(resGO);
}


/*
 * 0xa6-0xa7 2 24 MOV @Ri,addr
 *____________________________________________________________________________
 *
 */

int
t_uc::inst_mov_$ri_addr(uchar code)
{
  uchar *d, *s;
  int res;

  d= get_indirect(event_at.wi= *(get_reg(code & 0x07)), &res);
  s= get_direct(fetch(), &event_at.ri, &event_at.rs);
  (*d)= read(s);
  tick(1);
  return(res);
}


/*
 * 0xa8-0xaf 2 24 MOV Rn,addr
 *____________________________________________________________________________
 *
 */

int
t_uc::inst_mov_rn_addr(uchar code)
{
  uchar *reg, *addr;

  reg = get_reg(code & 0x07, &event_at.wi);
  addr= get_direct(fetch(), &event_at.ri, &event_at.rs);
  (*reg)= read(addr);
  tick(1);
  return(resGO);
}


/*
 * 0xc0 2 24 PUSH addr
 *____________________________________________________________________________
 *
 */

int
t_uc::inst_push(uchar code)
{
  uchar *addr, *sp;
  int res;

  addr= get_direct(fetch(), &event_at.ri, &event_at.rs);
  SFR[SP]++;
  sp= get_indirect(SFR[SP], &res);
  if (res != resGO)
    res= resSTACK_OV;
  (*sp)= read(addr);
  tick(1);
  return(res);
}


/*
 * 0xc5 2 12 XCH A,addr
 *____________________________________________________________________________
 *
 */

int
t_uc::inst_xch_a_addr(uchar code)
{
  uchar temp, *addr;

  addr= get_direct(fetch(), &event_at.ri, &event_at.rs);
  temp= SFR[ACC];
  SFR[event_at.ws= ACC]= read(addr);
  (*addr)= temp;
  proc_write(addr);
  return(resGO);
}


/*
 * 0xc6-0xc7 1 12 XCH A,@Ri
 *____________________________________________________________________________
 *
 */

int
t_uc::inst_xch_a_$ri(uchar code)
{
  uchar temp, *addr;
  int res;

  addr= get_indirect(event_at.ri= *(get_reg(code & 0x07)), &res);
  temp= SFR[ACC];
  SFR[event_at.ws= ACC]= *addr;
  (*addr)= temp;
  return(res);
}


/*
 * 0xc8-0xcf 1 12 XCH A,Rn
 *____________________________________________________________________________
 *
 */

int
t_uc::inst_xch_a_rn(uchar code)
{
  uchar temp, *reg;

  reg = get_reg(code & 0x07, &event_at.ri);
  temp= SFR[ACC];
  SFR[event_at.wi= ACC]= *reg;
  (*reg)= temp;
  return(resGO);
}


/*
 * 0xd0 2 24 POP addr
 *____________________________________________________________________________
 *
 */

int
t_uc::inst_pop(uchar code)
{
  uchar *addr, *sp;
  int res;

  addr= get_direct(fetch(), &event_at.wi, &event_at.ws);
  sp= get_indirect(SFR[SP], &res);
  if (res != resGO)
    res= resSTACK_OV;
  SFR[SP]--;
  (*addr)= *sp;
  tick(1);
  proc_write(addr);
  return(res);
}


/*
 * 0xd6-0xd7 1 12 XCHD A,@Ri
 *____________________________________________________________________________
 *
 */

int
t_uc::inst_xchd_a_$ri(uchar code)
{
  uchar *addr, temp;
  int res;

  addr= get_indirect(event_at.ri= *(get_reg(code & 0x01)), &res);
  temp= *addr & 0x0f;
  (*addr) = (*addr & 0xf0) | (SFR[ACC] & 0x0f);
  SFR[event_at.ws= ACC]= (SFR[ACC] & 0xf0) | temp;
  return(res);
}


/*
 * 0xe0 1 24 MOVX A,@DPTR
 *____________________________________________________________________________
 *
 */

int
t_uc::inst_movx_a_$dptr(uchar code)
{
  SFR[event_at.ws= ACC]= XRAM[event_at.rx= SFR[DPH]*256+SFR[DPL]];
  tick(1);
  return(resGO);
}


/*
 * 0xe2-0xe3 1 24 MOVX A,@Ri
 *____________________________________________________________________________
 *
 */

int
t_uc::inst_movx_a_$ri(uchar code)
{
  uchar *addr;
  int res;

  addr= get_indirect(*(get_reg(code & 0x01)), &res);
  SFR[event_at.ws= ACC]= XRAM[event_at.rx= (SFR[P2]&port_pins[2])*256+*addr];
  tick(1);
  return(res);
}


/*
 * 0xe5 2 12 MOV A,addr
 *____________________________________________________________________________
 *
 */

int
t_uc::inst_mov_a_addr(uchar code)
{
  uchar *addr;

  addr= get_direct(fetch(), &event_at.ri, &event_at.rs);
  SFR[event_at.ws= ACC]= read(addr);
  return(resGO);
}


/*
 * 0xe6-0xe7 1 12 MOV A,@Ri
 *____________________________________________________________________________
 *
 */

int
t_uc::inst_mov_a_$ri(uchar code)
{
  uchar *addr;
  int res;

  addr= get_indirect(event_at.ri= *(get_reg(code & 0x01)), &res);
  SFR[event_at.ws= ACC]= *addr;
  return(res);
}


/*
 * 0xe8-0xef 1 12 MOV A,Rn
 *____________________________________________________________________________
 *
 */

int
t_uc::inst_mov_a_rn(uchar code)
{
  SFR[event_at.ws= ACC]= *(get_reg(code & 0x07, &event_at.ri));
  return(resGO);
}


/*
 * 0xf0 1 24 MOVX @DPTR,A
 *____________________________________________________________________________
 *
 */

int
t_uc::inst_movx_$dptr_a(uchar code)
{
  XRAM[event_at.wx= SFR[DPH]*256+SFR[DPL]]= SFR[event_at.rs= ACC];
  return(resGO);
}


/*
 * 0xf2-0xf3 1 24 MOVX @Ri,A
 *____________________________________________________________________________
 *
 */

int
t_uc::inst_movx_$ri_a(uchar code)
{
  uchar *addr;
  int res;

  addr= get_indirect(event_at.wi= *(get_reg(code & 0x01)), &res);
  XRAM[event_at.wx= (SFR[P2] & port_pins[2])*256 + *addr]= SFR[ACC];
  tick(1);
  return(res);
}


/*
 * 0xf5 2 12 MOV addr,A
 *____________________________________________________________________________
 *
 */

int
t_uc::inst_mov_addr_a(uchar code)
{
  uchar *addr;

  addr= get_direct(fetch(), &event_at.wi, &event_at.ws);
  (*addr)= SFR[event_at.rs= ACC];
  proc_write(addr);
  return(resGO);
}


/*
 * 0xf6-0xf7 1 12 MOV @Ri,A
 *____________________________________________________________________________
 *
 */

int
t_uc::inst_mov_$ri_a(uchar code)
{
  uchar *addr;
  int res;

  addr= get_indirect(event_at.wi= *(get_reg(code & 0x01)), &res);
  (*addr)= SFR[event_at.rs= ACC];
  return(res);
}


/*
 * 0xf8-0xff 1 12 MOV Rn,A
 *____________________________________________________________________________
 *
 */

int
t_uc::inst_mov_rn_a(uchar code)
{
  uchar *reg;

  reg= get_reg(code &0x07, &event_at.wi);
  (*reg)= SFR[event_at.rs= ACC];
  return(resGO);
}


/* End of mov.cc */
